<?php

namespace Inventory\Bank\Policies;

use Litepie\User\Contracts\UserPolicy;
use Inventory\Bank\Models\Bank;

class BankPolicy
{

    /**
     * Determine if the given user can view the bank.
     *
     * @param UserPolicy $user
     * @param Bank $bank
     *
     * @return bool
     */
    public function view(UserPolicy $user, Bank $bank)
    {
        if ($user->canDo('bank.bank.view') && $user->isAdmin()) {
            return true;
        }

        return $bank->user_id == user_id() && $bank->user_type == user_type();
    }

    /**
     * Determine if the given user can create a bank.
     *
     * @param UserPolicy $user
     * @param Bank $bank
     *
     * @return bool
     */
    public function create(UserPolicy $user)
    {
        return  $user->canDo('bank.bank.create');
    }

    /**
     * Determine if the given user can update the given bank.
     *
     * @param UserPolicy $user
     * @param Bank $bank
     *
     * @return bool
     */
    public function update(UserPolicy $user, Bank $bank)
    {
        if ($user->canDo('bank.bank.edit') && $user->isAdmin()) {
            return true;
        }

        return $bank->user_id == user_id() && $bank->user_type == user_type();
    }

    /**
     * Determine if the given user can delete the given bank.
     *
     * @param UserPolicy $user
     * @param Bank $bank
     *
     * @return bool
     */
    public function destroy(UserPolicy $user, Bank $bank)
    {
        return $bank->user_id == user_id() && $bank->user_type == user_type();
    }

    /**
     * Determine if the given user can verify the given bank.
     *
     * @param UserPolicy $user
     * @param Bank $bank
     *
     * @return bool
     */
    public function verify(UserPolicy $user, Bank $bank)
    {
        if ($user->canDo('bank.bank.verify')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the given user can approve the given bank.
     *
     * @param UserPolicy $user
     * @param Bank $bank
     *
     * @return bool
     */
    public function approve(UserPolicy $user, Bank $bank)
    {
        if ($user->canDo('bank.bank.approve')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the user can perform a given action ve.
     *
     * @param [type] $user    [description]
     * @param [type] $ability [description]
     *
     * @return [type] [description]
     */
    public function before($user, $ability)
    {
        if ($user->isSuperuser()) {
            return true;
        }
    }
}
